@@ -14,13 +14,15 @@ namespace x86Emitter
 	//
 	struct xImplSimd_MovHL
 	{
-		u16 Opcode;
+		SIMDInstructionInfo info;
 
-		void PS(const xRegisterSSE& to, const xIndirectVoid& from) const;
-		void PS(const xIndirectVoid& to, const xRegisterSSE& from) const;
+		void PS(const xRegisterSSE& dst, const xIndirectVoid& src) const { PS(dst, dst, src); }
+		void PS(const xRegisterSSE& dst, const xRegisterSSE& src1, const xIndirectVoid& src2) const;
+		void PS(const xIndirectVoid& dst, const xRegisterSSE& src) const;
 
-		void PD(const xRegisterSSE& to, const xIndirectVoid& from) const;
-		void PD(const xIndirectVoid& to, const xRegisterSSE& from) const;
+		void PD(const xRegisterSSE& dst, const xIndirectVoid& src) const { PD(dst, dst, src); }
+		void PD(const xRegisterSSE& dst, const xRegisterSSE& src1, const xIndirectVoid& src2) const;
+		void PD(const xIndirectVoid& dst, const xRegisterSSE& src) const;
 	};
 
 	// --------------------------------------------------------------------------------------
@@ -31,16 +33,18 @@ namespace x86Emitter
 	//
 	struct xImplSimd_MovHL_RtoR
 	{
-		u16 Opcode;
+		SIMDInstructionInfo info;
 
-		void PS(const xRegisterSSE& to, const xRegisterSSE& from) const;
-		void PD(const xRegisterSSE& to, const xRegisterSSE& from) const;
+		void PS(const xRegisterSSE& dst, const xRegisterSSE& src) const { PS(dst, dst, src); }
+		void PD(const xRegisterSSE& dst, const xRegisterSSE& src) const { PD(dst, dst, src); }
+		void PS(const xRegisterSSE& dst, const xRegisterSSE& src1, const xRegisterSSE& src2) const;
+		void PD(const xRegisterSSE& dst, const xRegisterSSE& src1, const xRegisterSSE& src2) const;
 	};
 
 	// --------------------------------------------------------------------------------------
 	//  xImplSimd_MoveSSE
 	// --------------------------------------------------------------------------------------
-	// Legends in their own right: MOVAPS / MOVAPD / MOVUPS / MOVUPD
+	// Legends in their own right: MOVAPS / MOVAPD / MOVUPS / MOVUPD / MOVDQA / MOVDQU
 	//
 	// All implementations of Unaligned Movs will, when possible, use aligned movs instead.
 	// This happens when using Mem,Reg or Reg,Mem forms where the address is simple displacement
@@ -48,27 +52,10 @@ namespace x86Emitter
 	//
 	struct xImplSimd_MoveSSE
 	{
-		u8 Prefix;
-		bool isAligned;
-
-		void operator()(const xRegisterSSE& to, const xRegisterSSE& from) const;
-		void operator()(const xRegisterSSE& to, const xIndirectVoid& from) const;
-		void operator()(const xIndirectVoid& to, const xRegisterSSE& from) const;
-	};
-
-	// --------------------------------------------------------------------------------------
-	//  xImplSimd_MoveDQ
-	// --------------------------------------------------------------------------------------
-	// Implementations for MOVDQA / MOVDQU
-	//
-	// All implementations of Unaligned Movs will, when possible, use aligned movs instead.
-	// This happens when using Mem,Reg or Reg,Mem forms where the address is simple displacement
-	// which can be checked for alignment at runtime.
-
-	struct xImplSimd_MoveDQ
-	{
-		u8 Prefix;
-		bool isAligned;
+		SIMDInstructionInfo aligned_load;
+		SIMDInstructionInfo aligned_store;
+		SIMDInstructionInfo unaligned_load;
+		SIMDInstructionInfo unaligned_store;
 
 		void operator()(const xRegisterSSE& to, const xRegisterSSE& from) const;
 		void operator()(const xRegisterSSE& to, const xIndirectVoid& from) const;
@@ -89,7 +76,7 @@ namespace x86Emitter
 		// If a mask bit is 1, then the corresponding dword in the source operand is copied
 		// to dest, else the dword element in dest is left unchanged.
 		//
-		xImplSimd_DestRegImmSSE PS;
+		xImplSimd_3ArgImm PS;
 
 		// [SSE-4.1] Conditionally copies quadword values from src to dest, depending on the
 		// mask bits in the immediate operand (bits [1:0]).  Each mask bit corresponds to a
@@ -98,7 +85,7 @@ namespace x86Emitter
 		// If a mask bit is 1, then the corresponding dword in the source operand is copied
 		// to dest, else the dword element in dest is left unchanged.
 		//
-		xImplSimd_DestRegImmSSE PD;
+		xImplSimd_3ArgImm PD;
 
 		// [SSE-4.1] Conditionally copies dword values from src to dest, depending on the
 		// mask (bits [3:0]) in XMM0 (yes, the fixed register).  Each mask bit corresponds
@@ -107,7 +94,7 @@ namespace x86Emitter
 		// If a mask bit is 1, then the corresponding dword in the source operand is copied
 		// to dest, else the dword element in dest is left unchanged.
 		//
-		xImplSimd_DestRegSSE VPS;
+		xImplSimd_4ArgBlend VPS;
 
 		// [SSE-4.1] Conditionally copies quadword values from src to dest, depending on the
 		// mask (bits [1:0]) in XMM0 (yes, the fixed register).  Each mask bit corresponds
@@ -116,13 +103,13 @@ namespace x86Emitter
 		// If a mask bit is 1, then the corresponding dword in the source operand is copied
 		// to dest, else the dword element in dest is left unchanged.
 		//
-		xImplSimd_DestRegSSE VPD;
+		xImplSimd_4ArgBlend VPD;
 	};
 
 	struct xImplSimd_PBlend
 	{
-		xImplSimd_DestRegImmSSE W;
-		xImplSimd_DestRegSSE VB;
+		xImplSimd_3ArgImm W;
+		xImplSimd_4ArgBlend VB;
 	};
 
 	// --------------------------------------------------------------------------------------
@@ -132,36 +119,36 @@ namespace x86Emitter
 	//
 	struct xImplSimd_PMove
 	{
-		u16 OpcodeBase;
+		SIMDInstructionInfo info;
 
 		// [SSE-4.1] Zero/Sign-extend the low byte values in src into word integers
 		// and store them in dest.
 		void BW(const xRegisterSSE& to, const xRegisterSSE& from) const;
-		void BW(const xRegisterSSE& to, const xIndirect64& from) const;
+		void BW(const xRegisterSSE& to, const xIndirectVoid& from) const;
 
 		// [SSE-4.1] Zero/Sign-extend the low byte values in src into dword integers
 		// and store them in dest.
 		void BD(const xRegisterSSE& to, const xRegisterSSE& from) const;
-		void BD(const xRegisterSSE& to, const xIndirect32& from) const;
+		void BD(const xRegisterSSE& to, const xIndirectVoid& from) const;
 
 		// [SSE-4.1] Zero/Sign-extend the low byte values in src into qword integers
 		// and store them in dest.
 		void BQ(const xRegisterSSE& to, const xRegisterSSE& from) const;
-		void BQ(const xRegisterSSE& to, const xIndirect16& from) const;
+		void BQ(const xRegisterSSE& to, const xIndirectVoid& from) const;
 
 		// [SSE-4.1] Zero/Sign-extend the low word values in src into dword integers
 		// and store them in dest.
 		void WD(const xRegisterSSE& to, const xRegisterSSE& from) const;
-		void WD(const xRegisterSSE& to, const xIndirect64& from) const;
+		void WD(const xRegisterSSE& to, const xIndirectVoid& from) const;
 
 		// [SSE-4.1] Zero/Sign-extend the low word values in src into qword integers
 		// and store them in dest.
 		void WQ(const xRegisterSSE& to, const xRegisterSSE& from) const;
-		void WQ(const xRegisterSSE& to, const xIndirect32& from) const;
+		void WQ(const xRegisterSSE& to, const xIndirectVoid& from) const;
 
 		// [SSE-4.1] Zero/Sign-extend the low dword values in src into qword integers
 		// and store them in dest.
 		void DQ(const xRegisterSSE& to, const xRegisterSSE& from) const;
-		void DQ(const xRegisterSSE& to, const xIndirect64& from) const;
+		void DQ(const xRegisterSSE& to, const xIndirectVoid& from) const;
 	};
 } // namespace x86Emitter
