@@ -1,21 +1,13 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2014-2022  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
-#include "PrecompiledHeader.h"
 #include "MipsAssembler.h"
 
+#include <array>
+#include <algorithm>
+#include <cctype>
+#include <string_view>
+
 // just an empty class, so that it's not necessary to remove all the calls manually
 // will make it easier to update if there are changes later on
 class Logger
@@ -35,133 +27,176 @@ public:
 
 };
 
+constexpr auto MipsRegisters = std::to_array<tMipsRegister>({
+	{ "r0", 0},  { "zero", 0}, { "$0", 0 }, { "$zero", 0 },
+	{ "at", 1},  { "r1", 1},   { "$1", 1 }, { "$at", 1 },
+	{ "v0", 2},  { "r2", 2},   { "$v0", 2 },
+	{ "v1", 3},  { "r3", 3},   { "$v1", 3 },
+	{ "a0", 4},  { "r4", 4},   { "$a0", 4 },
+	{ "a1", 5},  { "r5", 5},   { "$a1", 5 },
+	{ "a2", 6},  { "r6", 6},   { "$a2", 6 },
+	{ "a3", 7},  { "r7", 7},   { "$a3", 7 },
+	{ "t0", 8},  { "r8", 8},   { "$t0", 8 },
+	{ "t1", 9},  { "r9", 9},   { "$t1", 9 },
+	{ "t2", 10}, { "r10", 10}, { "$t2", 10 },
+	{ "t3", 11}, { "r11", 11}, { "$t3", 11 },
+	{ "t4", 12}, { "r12", 12}, { "$t4", 12 },
+	{ "t5", 13}, { "r13", 13}, { "$t5", 13 },
+	{ "t6", 14}, { "r14", 14}, { "$t6", 14 },
+	{ "t7", 15}, { "r15", 15}, { "$t7", 15 },
+	{ "s0", 16}, { "r16", 16}, { "$s0", 16 },
+	{ "s1", 17}, { "r17", 17}, { "$s1", 17 },
+	{ "s2", 18}, { "r18", 18}, { "$s2", 18 },
+	{ "s3", 19}, { "r19", 19}, { "$s3", 19 },
+	{ "s4", 20}, { "r20", 20}, { "$s4", 20 },
+	{ "s5", 21}, { "r21", 21}, { "$s5", 21 },
+	{ "s6", 22}, { "r22", 22}, { "$s6", 22 },
+	{ "s7", 23}, { "r23", 23}, { "$s7", 23 },
+	{ "t8", 24}, { "r24", 24}, { "$t8", 24 },
+	{ "t9", 25}, { "r25", 25}, { "$t9", 25 },
+	{ "k0", 26}, { "r26", 26}, { "$k0", 26 },
+	{ "k1", 27}, { "r27", 27}, { "$k1", 27 },
+	{ "gp", 28}, { "r28", 28}, { "$gp", 28 },
+	{ "sp", 29}, { "r29", 29}, { "$sp", 29 },
+	{ "fp", 30}, { "r30", 30}, { "$fp", 30 },
+	{ "ra", 31}, { "r31", 31}, { "$ra", 31 }
+});
 
-const tMipsRegister MipsRegister[] = {
-	{ "r0", 0, 2 }, { "zero", 0, 4}, { "$0", 0, 2 }, { "$zero", 0, 5 },
-	{ "at", 1, 2 }, { "r1", 1, 2 }, { "$1", 1, 2 }, { "$at", 1, 3 },
-	{ "v0", 2, 2 }, { "r2", 2, 2 }, { "$v0", 2, 3 },
-	{ "v1", 3, 2 }, { "r3", 3, 2 }, { "$v1", 3, 3 },
-	{ "a0", 4, 2 }, { "r4", 4, 2 }, { "$a0", 4, 3 },
-	{ "a1", 5, 2 }, { "r5", 5, 2 }, { "$a1", 5, 3 },
-	{ "a2", 6, 2 }, { "r6", 6, 2 }, { "$a2", 6, 3 },
-	{ "a3", 7, 2 }, { "r7", 7, 2 }, { "$a3", 7, 3 },
-	{ "t0", 8, 2 }, { "r8", 8, 2 }, { "$t0", 8, 3 },
-	{ "t1", 9, 2 }, { "r9", 9, 2 }, { "$t1", 9, 3 },
-	{ "t2", 10, 2 }, { "r10", 10, 3 }, { "$t2", 10, 3 },
-	{ "t3", 11, 2 }, { "r11", 11, 3 }, { "$t3", 11, 3 },
-	{ "t4", 12, 2 }, { "r12", 12, 3 }, { "$t4", 12, 3 },
-	{ "t5", 13, 2 }, { "r13", 13, 3 }, { "$t5", 13, 3 },
-	{ "t6", 14, 2 }, { "r14", 14, 3 }, { "$t6", 14, 3 },
-	{ "t7", 15, 2 }, { "r15", 15, 3 }, { "$t7", 15, 3 },
-	{ "s0", 16, 2 }, { "r16", 16, 3 }, { "$s0", 16, 3 },
-	{ "s1", 17, 2 }, { "r17", 17, 3 }, { "$s1", 17, 3 },
-	{ "s2", 18, 2 }, { "r18", 18, 3 }, { "$s2", 18, 3 },
-	{ "s3", 19, 2 }, { "r19", 19, 3 }, { "$s3", 19, 3 },
-	{ "s4", 20, 2 }, { "r20", 20, 3 }, { "$s4", 20, 3 },
-	{ "s5", 21, 2 }, { "r21", 21, 3 }, { "$s5", 21, 3 },
-	{ "s6", 22, 2 }, { "r22", 22, 3 }, { "$s6", 22, 3 },
-	{ "s7", 23, 2 }, { "r23", 23, 3 }, { "$s7", 23, 3 },
-	{ "t8", 24, 2 }, { "r24", 24, 3 }, { "$t8", 24, 3 },
-	{ "t9", 25, 2 }, { "r25", 25, 3 }, { "$t9", 25, 3 },
-	{ "k0", 26, 2 }, { "r26", 26, 3 }, { "$k0", 26, 3 },
-	{ "k1", 27, 2 }, { "r27", 27, 3 }, { "$k1", 27, 3 },
-	{ "gp", 28, 2 }, { "r28", 28, 3 }, { "$gp", 28, 3 },
-	{ "sp", 29, 2 }, { "r29", 29, 3 }, { "$sp", 29, 3 },
-	{ "fp", 30, 2 }, { "r30", 30, 3 }, { "$fp", 30, 3 },
-	{ "ra", 31, 2 }, { "r31", 31, 3 }, { "$ra", 31, 3 },
-	{ NULL, -1, 0}
-};
+constexpr auto MipsFloatRegisters = std::to_array<tMipsRegister>({
+	{ "f0", 0, },	{ "$f0", 0 },
+	{ "f1", 1, },	{ "$f1", 1 },
+	{ "f2", 2, },	{ "$f2", 2 },
+	{ "f3", 3, },	{ "$f3", 3 },
+	{ "f4", 4, },	{ "$f4", 4 },
+	{ "f5", 5, },	{ "$f5", 5 },
+	{ "f6", 6, },	{ "$f6", 6 },
+	{ "f7", 7, },	{ "$f7", 7 },
+	{ "f8", 8, },	{ "$f8", 8 },
+	{ "f9", 9, },	{ "$f9", 9 },
+	{ "f00", 0,},	{ "$f00", 0 },
+	{ "f01", 1,},	{ "$f01", 1 },
+	{ "f02", 2,},	{ "$f02", 2 },
+	{ "f03", 3,},	{ "$f03", 3 },
+	{ "f04", 4,},	{ "$f04", 4 },
+	{ "f05", 5,},	{ "$f05", 5 },
+	{ "f06", 6,},	{ "$f06", 6 },
+	{ "f07", 7,},	{ "$f07", 7 },
+	{ "f08", 8,},	{ "$f08", 8 },
+	{ "f09", 9,},	{ "$f09", 9 },
+	{ "f10", 10 },	{ "$f10", 10 },
+	{ "f11", 11 },	{ "$f11", 11 },
+	{ "f12", 12 },	{ "$f12", 12 },
+	{ "f13", 13 },	{ "$f13", 13 },
+	{ "f14", 14 },	{ "$f14", 14 },
+	{ "f15", 15 },	{ "$f15", 15 },
+	{ "f16", 16 },	{ "$f16", 16 },
+	{ "f17", 17 },	{ "$f17", 17 },
+	{ "f18", 18 },	{ "$f18", 18 },
+	{ "f19", 19 },	{ "$f19", 19 },
+	{ "f20", 20 },	{ "$f20", 20 },
+	{ "f21", 21 },	{ "$f21", 21 },
+	{ "f22", 22 },	{ "$f22", 22 },
+	{ "f23", 23 },	{ "$f23", 23 },
+	{ "f24", 24 },	{ "$f24", 24 },
+	{ "f25", 25 },	{ "$f25", 25 },
+	{ "f26", 26 },	{ "$f26", 26 },
+	{ "f27", 27 },	{ "$f27", 27 },
+	{ "f28", 28 },	{ "$f28", 28 },
+	{ "f29", 29 },	{ "$f29", 29 },
+	{ "f30", 30 },	{ "$f30", 30 },
+	{ "f31", 31 },	{ "$f31", 31 }
+});
 
+constexpr auto MipsPs2Cop2FpRegisters = std::to_array<tMipsRegister>({
+	{ "vf0", 0 },	{ "$vf0", 0 },
+	{ "vf1", 1 },	{ "$vf1", 1 },
+	{ "vf2", 2 },	{ "$vf2", 2 },
+	{ "vf3", 3 },	{ "$vf3", 3 },
+	{ "vf4", 4 },	{ "$vf4", 4 },
+	{ "vf5", 5 },	{ "$vf5", 5 },
+	{ "vf6", 6 },	{ "$vf6", 6 },
+	{ "vf7", 7 },	{ "$vf7", 7 },
+	{ "vf8", 8 },	{ "$vf8", 8 },
+	{ "vf9", 9 },	{ "$vf9", 9 },
+	{ "vf00", 0 },	{ "$vf00", 0 },
+	{ "vf01", 1 },	{ "$vf01", 1 },
+	{ "vf02", 2 },	{ "$vf02", 2 },
+	{ "vf03", 3 },	{ "$vf03", 3 },
+	{ "vf04", 4 },	{ "$vf04", 4 },
+	{ "vf05", 5 },	{ "$vf05", 5 },
+	{ "vf06", 6 },	{ "$vf06", 6 },
+	{ "vf07", 7 },	{ "$vf07", 7 },
+	{ "vf08", 8 },	{ "$vf08", 8 },
+	{ "vf09", 9 },	{ "$vf09", 9 },
+	{ "vf10", 10 },	{ "$vf10", 10 },
+	{ "vf11", 11 },	{ "$vf11", 11 },
+	{ "vf12", 12 },	{ "$vf12", 12 },
+	{ "vf13", 13 },	{ "$vf13", 13 },
+	{ "vf14", 14 },	{ "$vf14", 14 },
+	{ "vf15", 15 },	{ "$vf15", 15 },
+	{ "vf16", 16 },	{ "$vf16", 16 },
+	{ "vf17", 17 },	{ "$vf17", 17 },
+	{ "vf18", 18 },	{ "$vf18", 18 },
+	{ "vf19", 19 },	{ "$vf19", 19 },
+	{ "vf20", 20 },	{ "$vf20", 20 },
+	{ "vf21", 21 },	{ "$vf21", 21 },
+	{ "vf22", 22 },	{ "$vf22", 22 },
+	{ "vf23", 23 },	{ "$vf23", 23 },
+	{ "vf24", 24 },	{ "$vf24", 24 },
+	{ "vf25", 25 },	{ "$vf25", 25 },
+	{ "vf26", 26 },	{ "$vf26", 26 },
+	{ "vf27", 27 },	{ "$vf27", 27 },
+	{ "vf28", 28 },	{ "$vf28", 28 },
+	{ "vf29", 29 },	{ "$vf29", 29 },
+	{ "vf30", 30 },	{ "$vf30", 30 },
+	{ "vf31", 31 },	{ "$vf31", 31 }
+});
 
-const tMipsRegister MipsFloatRegister[] = {
-	{ "f0", 0, 2},		{ "$f0", 0, 3 },
-	{ "f1", 1, 2},		{ "$f1", 1, 3 },
-	{ "f2", 2, 2},		{ "$f2", 2, 3 },
-	{ "f3", 3, 2},		{ "$f3", 3, 3 },
-	{ "f4", 4, 2},		{ "$f4", 4, 3 },
-	{ "f5", 5, 2},		{ "$f5", 5, 3 },
-	{ "f6", 6, 2},		{ "$f6", 6, 3 },
-	{ "f7", 7, 2},		{ "$f7", 7, 3 },
-	{ "f8", 8, 2},		{ "$f8", 8, 3 },
-	{ "f9", 9, 2},		{ "$f9", 9, 3 },
-	{ "f00", 0, 3},		{ "$f00", 0, 4 },
-	{ "f01", 1, 3},		{ "$f01", 1, 4 },
-	{ "f02", 2, 3},		{ "$f02", 2, 4 },
-	{ "f03", 3, 3},		{ "$f03", 3, 4 },
-	{ "f04", 4, 3},		{ "$f04", 4, 4 },
-	{ "f05", 5, 3},		{ "$f05", 5, 4 },
-	{ "f06", 6, 3},		{ "$f06", 6, 4 },
-	{ "f07", 7, 3},		{ "$f07", 7, 4 },
-	{ "f08", 8, 3},		{ "$f08", 8, 4 },
-	{ "f09", 9, 3},		{ "$f09", 9, 4 },
-	{ "f10", 10, 3},	{ "$f10", 10, 4 },
-	{ "f11", 11, 3},	{ "$f11", 11, 4 },
-	{ "f12", 12, 3},	{ "$f12", 12, 4 },
-	{ "f13", 13, 3},	{ "$f13", 13, 4 },
-	{ "f14", 14, 3},	{ "$f14", 14, 4 },
-	{ "f15", 15, 3},	{ "$f15", 15, 4 },
-	{ "f16", 16, 3},	{ "$f16", 16, 4 },
-	{ "f17", 17, 3},	{ "$f17", 17, 4 },
-	{ "f18", 18, 3},	{ "$f18", 18, 4 },
-	{ "f19", 19, 3},	{ "$f19", 19, 4 },
-	{ "f20", 20, 3},	{ "$f20", 20, 4 },
-	{ "f21", 21, 3},	{ "$f21", 21, 4 },
-	{ "f22", 22, 3},	{ "$f22", 22, 4 },
-	{ "f23", 23, 3},	{ "$f23", 23, 4 },
-	{ "f24", 24, 3},	{ "$f24", 24, 4 },
-	{ "f25", 25, 3},	{ "$f25", 25, 4 },
-	{ "f26", 26, 3},	{ "$f26", 26, 4 },
-	{ "f27", 27, 3},	{ "$f27", 27, 4 },
-	{ "f28", 28, 3},	{ "$f28", 28, 4 },
-	{ "f29", 29, 3},	{ "$f29", 29, 4 },
-	{ "f30", 30, 3},	{ "$f30", 30, 4 },
-	{ "f31", 31, 3},	{ "$f31", 31, 4 }
-};
+constexpr auto MipsPs2Cop2IRegisters = std::to_array<tMipsRegister>({
+	{ "vi0", 0 },	{ "$vi0", 0 },
+	{ "vi1", 1 },	{ "$vi1", 1 },
+	{ "vi2", 2 },	{ "$vi2", 2 },
+	{ "vi3", 3 },	{ "$vi3", 3 },
+	{ "vi4", 4 },	{ "$vi4", 4 },
+	{ "vi5", 5 },	{ "$vi5", 5 },
+	{ "vi6", 6 },	{ "$vi6", 6 },
+	{ "vi7", 7 },	{ "$vi7", 7 },
+	{ "vi8", 8 },	{ "$vi8", 8 },
+	{ "vi9", 9 },	{ "$vi9", 9 },
+	{ "vi00", 0 },	{ "$vi00", 0 },
+	{ "vi01", 1 },	{ "$vi01", 1 },
+	{ "vi02", 2 },	{ "$vi02", 2 },
+	{ "vi03", 3 },	{ "$vi03", 3 },
+	{ "vi04", 4 },	{ "$vi04", 4 },
+	{ "vi05", 5 },	{ "$vi05", 5 },
+	{ "vi06", 6 },	{ "$vi06", 6 },
+	{ "vi07", 7 },	{ "$vi07", 7 },
+	{ "vi08", 8 },	{ "$vi08", 8 },
+	{ "vi09", 9 },	{ "$vi09", 9 },
+	{ "vi10", 10 },	{ "$vi10", 10 },
+	{ "vi11", 11 },	{ "$vi11", 11 },
+	{ "vi12", 12 },	{ "$vi12", 12 },
+	{ "vi13", 13 },	{ "$vi13", 13 },
+	{ "vi14", 14 },	{ "$vi14", 14 },
+	{ "vi15", 15 },	{ "$vi15", 15 }
+});
 
-const tMipsRegister MipsPs2Cop2FpRegister[] = {
-	{ "vf0", 0, 3},		{ "$vf0", 0, 4 },
-	{ "vf1", 1, 3},		{ "$vf1", 1, 4 },
-	{ "vf2", 2, 3},		{ "$vf2", 2, 4 },
-	{ "vf3", 3, 3},		{ "$vf3", 3, 4 },
-	{ "vf4", 4, 3},		{ "$vf4", 4, 4 },
-	{ "vf5", 5, 3},		{ "$vf5", 5, 4 },
-	{ "vf6", 6, 3},		{ "$vf6", 6, 4 },
-	{ "vf7", 7, 3},		{ "$vf7", 7, 4 },
-	{ "vf8", 8, 3},		{ "$vf8", 8, 4 },
-	{ "vf9", 9, 3},		{ "$vf9", 9, 4 },
-	{ "vf00", 0, 4},	{ "$vf00", 0, 5 },
-	{ "vf01", 1, 4},	{ "$vf01", 1, 5 },
-	{ "vf02", 2, 4},	{ "$vf02", 2, 5 },
-	{ "vf03", 3, 4},	{ "$vf03", 3, 5 },
-	{ "vf04", 4, 4},	{ "$vf04", 4, 5 },
-	{ "vf05", 5, 4},	{ "$vf05", 5, 5 },
-	{ "vf06", 6, 4},	{ "$vf06", 6, 5 },
-	{ "vf07", 7, 4},	{ "$vf07", 7, 5 },
-	{ "vf08", 8, 4},	{ "$vf08", 8, 5 },
-	{ "vf09", 9, 4},	{ "$vf09", 9, 5 },
-	{ "vf10", 10, 4},	{ "$vf10", 10, 5 },
-	{ "vf11", 11, 4},	{ "$vf11", 11, 5 },
-	{ "vf12", 12, 4},	{ "$vf12", 12, 5 },
-	{ "vf13", 13, 4},	{ "$vf13", 13, 5 },
-	{ "vf14", 14, 4},	{ "$vf14", 14, 5 },
-	{ "vf15", 15, 4},	{ "$vf15", 15, 5 },
-	{ "vf16", 16, 4},	{ "$vf16", 16, 5 },
-	{ "vf17", 17, 4},	{ "$vf17", 17, 5 },
-	{ "vf18", 18, 4},	{ "$vf18", 18, 5 },
-	{ "vf19", 19, 4},	{ "$vf19", 19, 5 },
-	{ "vf20", 20, 4},	{ "$vf20", 20, 5 },
-	{ "vf21", 21, 4},	{ "$vf21", 21, 5 },
-	{ "vf22", 22, 4},	{ "$vf22", 22, 5 },
-	{ "vf23", 23, 4},	{ "$vf23", 23, 5 },
-	{ "vf24", 24, 4},	{ "$vf24", 24, 5 },
-	{ "vf25", 25, 4},	{ "$vf25", 25, 5 },
-	{ "vf26", 26, 4},	{ "$vf26", 26, 5 },
-	{ "vf27", 27, 4},	{ "$vf27", 27, 5 },
-	{ "vf28", 28, 4},	{ "$vf28", 28, 5 },
-	{ "vf29", 29, 4},	{ "$vf29", 29, 5 },
-	{ "vf30", 30, 4},	{ "$vf30", 30, 5 },
-	{ "vf31", 31, 4},	{ "$vf31", 31, 5 }
-};
+bool charEquals(char a, char b)
+{
+	return std::tolower(a) == std::tolower(b);
+}
+
+bool strEquals(std::string_view lhs, std::string_view rhs)
+{
+	const size_t compare_to = std::min(lhs.size(), rhs.size());
+	return std::ranges::equal(lhs.begin(), lhs.begin() +  compare_to, rhs.begin(), rhs.begin() + compare_to, charEquals);
+}
+
+bool isValidRegisterTrail(std::string_view source, size_t idx)
+{
+	return source.size() <= idx || source[idx] == ',' || source[idx] == '\n'  || source[idx] == 0
+	|| source[idx] == ')'  || source[idx] == '(' || source[idx] == '-';
+}
 
 void SplitLine(const char* Line, char* Name, char* Arguments)
 {
@@ -211,102 +246,29 @@ bool MipsAssembleOpcode(const char* line, DebugInterface* cpu, u32 address, u32&
 	return true;
 }
 
-
-bool MipsGetRegister(const char* source, int& RetLen, MipsRegisterInfo& Result)
+template<std::size_t SIZE>
+bool MipsGetRegister(const char* source, int& RetLen, MipsRegisterInfo& Result, const std::array<tMipsRegister, SIZE>& RegisterList)
 {
-	for (int z = 0; MipsRegister[z].name != NULL; z++)
-	{
-		int len = MipsRegister[z].len;
-		if (strncmp(MipsRegister[z].name,source,len) == 0)	// okay so far
+	for (const auto& reg : RegisterList) {
+		if(strEquals(reg.name, source) && isValidRegisterTrail(source, strlen(reg.name)))
 		{
-			if (source[len] == ',' || source[len] == '\n'  || source[len] == 0
-				|| source[len] == ')'  || source[len] == '(' || source[len] == '-')	// one of these has to come after a register
-			{
-				memcpy(Result.name,source,len);
-				Result.name[len] = 0;
-				Result.num = MipsRegister[z].num;
-				RetLen = len;
-				return true;
-			}
+			std::strncpy(Result.name, source, strlen(reg.name));
+			Result.num = reg.num;
+			RetLen = static_cast<int>(strlen(reg.name)); // grrr... should be unsigned!!
+			return true;
 		}
 	}
 	return false;
 }
 
-int MipsGetRegister(const char* source, int& RetLen)
+template<std::size_t SIZE>
+int MipsGetRegister(const char* source, int& RetLen, const std::array<tMipsRegister, SIZE>& RegisterList)
 {
-	for (int z = 0; MipsRegister[z].name != NULL; z++)
-	{
-		int len = MipsRegister[z].len;
-		if (strncmp(MipsRegister[z].name,source,len) == 0)	// okay so far
+	for (const auto& reg : RegisterList) {
+		if(strEquals(reg.name, source) && isValidRegisterTrail(source, strlen(reg.name)))
 		{
-			if (source[len] == ',' || source[len] == '\n'  || source[len] == 0
-				|| source[len] == ')'  || source[len] == '(' || source[len] == '-')	// one of these has to come after a register
-			{
-				RetLen = len;
-				return MipsRegister[z].num;
-			}
-		}
-	}
-	return -1;
-}
-
-
-bool MipsGetFloatRegister(const char* source, int& RetLen, MipsRegisterInfo& Result)
-{
-	for (int z = 0; MipsFloatRegister[z].name != NULL; z++)
-	{
-		int len = MipsFloatRegister[z].len;
-		if (strncmp(MipsFloatRegister[z].name,source,len) == 0)	// okay so far
-		{
-			if (source[len] == ',' || source[len] == '\n'  || source[len] == 0
-				|| source[len] == ')'  || source[len] == '(' || source[len] == '-')	// one of these has to come after a register
-			{
-				memcpy(Result.name,source,len);
-				Result.name[len] = 0;
-				Result.num = MipsFloatRegister[z].num;
-				RetLen = len;
-				return true;
-			}
-		}
-	}
-	return false;
-}
-
-bool MipsGetPs2VectorRegister(const char* source, int& RetLen, MipsRegisterInfo& Result)
-{
-	for (int z = 0; MipsPs2Cop2FpRegister[z].name != NULL; z++)
-	{
-		int len = MipsPs2Cop2FpRegister[z].len;
-		if (strncmp(MipsPs2Cop2FpRegister[z].name,source,len) == 0)	// okay so far
-		{
-			if (source[len] == ',' || source[len] == '\n'  || source[len] == 0
-				|| source[len] == ')'  || source[len] == '(' || source[len] == '-')	// one of these has to come after a register
-			{
-				memcpy(Result.name,source,len);
-				Result.name[len] = 0;
-				Result.num = MipsPs2Cop2FpRegister[z].num;
-				RetLen = len;
-				return true;
-			}
-		}
-	}
-	return false;
-}
-
-int MipsGetFloatRegister(const char* source, int& RetLen)
-{
-	for (int z = 0; MipsFloatRegister[z].name != NULL; z++)
-	{
-		int len = MipsFloatRegister[z].len;
-		if (strncmp(MipsFloatRegister[z].name,source,len) == 0)	// okay so far
-		{
-			if (source[len] == ',' || source[len] == '\n'  || source[len] == 0
-				|| source[len] == ')'  || source[len] == '(' || source[len] == '-')	// one of these has to come after a register
-			{
-				RetLen = len;
-				return MipsFloatRegister[z].num;
-			}
+			RetLen = static_cast<int>(strlen(reg.name)); // grrr... should be unsigned!!
+			return reg.num;
 		}
 	}
 	return -1;
@@ -319,7 +281,7 @@ bool MipsCheckImmediate(const char* Source, DebugInterface* cpu, int& dest, int&
 	int BufferPos = 0;
 	int l;
 
-	if (MipsGetRegister(Source,l) != -1)	// error
+	if (MipsGetRegister(Source,l, MipsRegisters) != -1)	// error
 	{
 		return false;
 	}
@@ -352,7 +314,7 @@ bool MipsCheckImmediate(const char* Source, DebugInterface* cpu, int& dest, int&
 
 		if (*Source == '(')	// could be part of the opcode
 		{
-			if (MipsGetRegister(Source+1,l) != -1)	// end
+			if (MipsGetRegister(Source+1,l, MipsRegisters) != -1)	// end
 			{
 				Buffer[BufferPos] = 0;
 				break;
@@ -366,11 +328,12 @@ bool MipsCheckImmediate(const char* Source, DebugInterface* cpu, int& dest, int&
 	RetLen = SourceLen;
 
 	PostfixExpression postfix;
-	if (!cpu->initExpression(Buffer,postfix))
+	std::string error;
+	if (!cpu->initExpression(Buffer,postfix,error))
 		return false;
 
 	u64 value;
-	if (!cpu->parseExpression(postfix,value))
+	if (!cpu->parseExpression(postfix,value,error))
 		return false;
 
 	dest = (int) value;
@@ -498,32 +461,32 @@ bool CMipsInstruction::LoadEncoding(const tMipsOpcode& SourceOpcode, const char*
 			switch (*SourceEncoding)
 			{
 			case 'T':	// float reg
-				if (!MipsGetFloatRegister(Line,RetLen,registers.frt)) return false;
+				if (!MipsGetRegister(Line,RetLen,registers.frt, MipsFloatRegisters)) return false;
 				Line += RetLen;
 				SourceEncoding++;
 				break;
 			case 'D':	// float reg
-				if (!MipsGetFloatRegister(Line,RetLen,registers.frd)) return false;
+				if (!MipsGetRegister(Line,RetLen,registers.frd, MipsFloatRegisters)) return false;
 				Line += RetLen;
 				SourceEncoding++;
 				break;
 			case 'S':	// float reg
-				if (!MipsGetFloatRegister(Line,RetLen,registers.frs)) return false;
+				if (!MipsGetRegister(Line,RetLen,registers.frs, MipsFloatRegisters)) return false;
 				Line += RetLen;
 				SourceEncoding++;
 				break;
 			case 't':
-				if (!MipsGetRegister(Line,RetLen,registers.grt)) return false;
+				if (!MipsGetRegister(Line,RetLen,registers.grt, MipsRegisters)) return false;
 				Line += RetLen;
 				SourceEncoding++;
 				break;
 			case 'd':
-				if (!MipsGetRegister(Line,RetLen,registers.grd)) return false;
+				if (!MipsGetRegister(Line,RetLen,registers.grd, MipsRegisters)) return false;
 				Line += RetLen;
 				SourceEncoding++;
 				break;
 			case 's':
-				if (!MipsGetRegister(Line,RetLen,registers.grs)) return false;
+				if (!MipsGetRegister(Line,RetLen,registers.grs, MipsRegisters)) return false;
 				Line += RetLen;
 				SourceEncoding++;
 				break;
@@ -531,17 +494,35 @@ bool CMipsInstruction::LoadEncoding(const tMipsOpcode& SourceOpcode, const char*
 				switch (*(SourceEncoding+1))
 				{
 				case 's':
-					if (!MipsGetPs2VectorRegister(Line,RetLen,registers.ps2vrs)) return false;
+					if (!MipsGetRegister(Line,RetLen,registers.ps2vrs, MipsPs2Cop2FpRegisters)) return false;
 					Line += RetLen;
 					break;
 				case 't':
-					if (!MipsGetPs2VectorRegister(Line,RetLen,registers.ps2vrt)) return false;
+					if (!MipsGetRegister(Line,RetLen,registers.ps2vrt, MipsPs2Cop2FpRegisters)) return false;
 					Line += RetLen;
 					break;
 				case 'd':
-					if (!MipsGetPs2VectorRegister(Line,RetLen,registers.ps2vrd)) return false;
+					if (!MipsGetRegister(Line,RetLen,registers.ps2vrd, MipsPs2Cop2FpRegisters)) return false;
 					Line += RetLen;
 					break;
+				case 'i':
+					switch(*(SourceEncoding+2))
+					{
+						case 's':
+							if (!MipsGetRegister(Line,RetLen,registers.ps2vrs, MipsPs2Cop2IRegisters)) return false;
+							Line += RetLen;
+							break;
+						case 't':
+							if (!MipsGetRegister(Line,RetLen,registers.ps2vrt, MipsPs2Cop2IRegisters)) return false;
+							Line += RetLen;
+							break;
+						case 'd':
+							if (!MipsGetRegister(Line,RetLen,registers.ps2vrd, MipsPs2Cop2IRegisters)) return false;
+							Line += RetLen;
+							break;
+					}
+					SourceEncoding += 1;
+					break;
 				default:
 					return false;
 				}
@@ -572,7 +553,7 @@ bool CMipsInstruction::LoadEncoding(const tMipsOpcode& SourceOpcode, const char*
 				SourceEncoding++;
 				break;
 			case 'r':	// forced register
-				if (MipsGetRegister(Line,RetLen) != *(SourceEncoding+1)) return false;
+				if (MipsGetRegister(Line,RetLen, MipsRegisters) != *(SourceEncoding+1)) return false;
 				Line += RetLen;
 				SourceEncoding += 2;
 				break;
@@ -656,14 +637,14 @@ bool CMipsInstruction::Validate()
 			immediate.value = (immediate.value >> 2) & 0x3FFFFFF;
 		} else if (Opcode.flags & MO_IPCR)	// relative 16 bit value
 		{
-			int num = (immediate.value-RamPos-4);
+			const int num = (immediate.value-RamPos-4) >> 2;
 
-			if (num > 0x20000 || num < (-0x20000))
+			if (num > std::numeric_limits<short>::max() || num < std::numeric_limits<short>::min())
 			{
 				Logger::queueError(Logger::Error,L"Branch target %08X out of range",immediate.value);
 				return false;
 			}
-			immediate.value = num >> 2;
+			immediate.value = num;
 		}
 
 		int immediateBits = getImmediateBits(immediateType);
@@ -695,7 +676,7 @@ void CMipsInstruction::encodeNormal()
 	if (registers.frd.num != -1) encoding |= MIPS_FD(registers.frd.num);	// float dest reg
 
 	if (registers.ps2vrt.num != -1) encoding |= (registers.ps2vrt.num << 16);	// ps2 vector target reg
-	if (registers.ps2vrs.num != -1) encoding |= (registers.ps2vrs.num << 21);	// ps2 vector source reg
+	if (registers.ps2vrs.num != -1) encoding |= (registers.ps2vrs.num << 11);	// ps2 vector source reg
 	if (registers.ps2vrd.num != -1) encoding |= (registers.ps2vrd.num << 6);	// ps2 vector dest reg
 
 	switch (immediateType)
