@@ -15,14 +15,19 @@
 #include "D3D12MemAlloc.h"
 
 GSTexture12::GSTexture12(Type type, Format format, int width, int height, int levels, DXGI_FORMAT dxgi_format,
-	wil::com_ptr_nothrow<ID3D12Resource> resource, wil::com_ptr_nothrow<D3D12MA::Allocation> allocation,
-	const D3D12DescriptorHandle& srv_descriptor, const D3D12DescriptorHandle& write_descriptor,
-	const D3D12DescriptorHandle& uav_descriptor, WriteDescriptorType wdtype, D3D12_RESOURCE_STATES resource_state)
+	wil::com_ptr_nothrow<ID3D12Resource> resource, wil::com_ptr_nothrow<ID3D12Resource> resource_fbl,
+	wil::com_ptr_nothrow<D3D12MA::Allocation> allocation, const D3D12DescriptorHandle& srv_descriptor,
+	const D3D12DescriptorHandle& write_descriptor, const D3D12DescriptorHandle& ro_dsv_descriptor,
+	const D3D12DescriptorHandle& uav_descriptor, const D3D12DescriptorHandle& fbl_descriptor,
+	WriteDescriptorType wdtype, D3D12_RESOURCE_STATES resource_state)
 	: m_resource(std::move(resource))
+	, m_resource_fbl(std::move(resource_fbl))
 	, m_allocation(std::move(allocation))
 	, m_srv_descriptor(srv_descriptor)
 	, m_write_descriptor(write_descriptor)
+	, m_read_dsv_descriptor(ro_dsv_descriptor)
 	, m_uav_descriptor(uav_descriptor)
+	, m_fbl_descriptor(fbl_descriptor)
 	, m_write_descriptor_type(wdtype)
 	, m_dxgi_format(dxgi_format)
 	, m_resource_state(resource_state)
@@ -55,6 +60,7 @@ void GSTexture12::Destroy(bool defer)
 				break;
 			case WriteDescriptorType::DSV:
 				dev->DeferDescriptorDestruction(dev->GetDSVHeapManager(), &m_write_descriptor);
+				dev->DeferDescriptorDestruction(dev->GetDSVHeapManager(), &m_read_dsv_descriptor);
 				break;
 			case WriteDescriptorType::None:
 			default:
@@ -64,8 +70,13 @@ void GSTexture12::Destroy(bool defer)
 		if (m_uav_descriptor)
 			dev->DeferDescriptorDestruction(dev->GetDescriptorHeapManager(), &m_uav_descriptor);
 
+		if (m_fbl_descriptor)
+			dev->DeferDescriptorDestruction(dev->GetDescriptorHeapManager(), &m_fbl_descriptor);
+
 		dev->DeferResourceDestruction(m_allocation.get(), m_resource.get());
+		dev->DeferResourceDestruction(m_allocation.get(), m_resource_fbl.get());
 		m_resource.reset();
+		m_resource_fbl.reset();
 		m_allocation.reset();
 	}
 	else
@@ -79,6 +90,7 @@ void GSTexture12::Destroy(bool defer)
 				break;
 			case WriteDescriptorType::DSV:
 				dev->GetDSVHeapManager().Free(&m_write_descriptor);
+				dev->GetDSVHeapManager().Free(&m_read_dsv_descriptor);
 				break;
 			case WriteDescriptorType::None:
 			default:
@@ -88,7 +100,11 @@ void GSTexture12::Destroy(bool defer)
 		if (m_uav_descriptor)
 			dev->GetDescriptorHeapManager().Free(&m_uav_descriptor);
 
+		if (m_fbl_descriptor)
+			dev->GetDescriptorHeapManager().Free(&m_fbl_descriptor);
+
 		m_resource.reset();
+		m_resource_fbl.reset();
 		m_allocation.reset();
 	}
 
@@ -125,7 +141,7 @@ std::unique_ptr<GSTexture12> GSTexture12::Create(Type type, Format format, int w
 			// This is a little annoying. basically, to do mipmap generation, we need to be a render target.
 			// If it's a compressed texture, we won't be generating mips anyway, so this should be fine.
 			desc.Flags = (levels > 1 && !IsCompressedFormat(format)) ? D3D12_RESOURCE_FLAG_ALLOW_RENDER_TARGET :
-																	   D3D12_RESOURCE_FLAG_NONE;
+			                                                           D3D12_RESOURCE_FLAG_NONE;
 			state = D3D12_RESOURCE_STATE_COPY_DEST;
 		}
 		break;
@@ -135,7 +151,9 @@ std::unique_ptr<GSTexture12> GSTexture12::Create(Type type, Format format, int w
 			// RT's tend to be larger, so we'll keep them committed for speed.
 			pxAssert(levels == 1);
 			allocationDesc.Flags |= D3D12MA::ALLOCATION_FLAG_COMMITTED;
-			desc.Flags = D3D12_RESOURCE_FLAG_ALLOW_RENDER_TARGET;
+			allocationDesc.ExtraHeapFlags = D3D12_HEAP_FLAG_DENY_BUFFERS | D3D12_HEAP_FLAG_DENY_NON_RT_DS_TEXTURES;
+			desc.Flags = D3D12_RESOURCE_FLAG_ALLOW_RENDER_TARGET | D3D12_RESOURCE_FLAG_ALLOW_SIMULTANEOUS_ACCESS;
+			desc.Layout = D3D12_TEXTURE_LAYOUT_64KB_UNDEFINED_SWIZZLE;
 			optimized_clear_value.Format = rtv_format;
 			state = D3D12_RESOURCE_STATE_RENDER_TARGET;
 		}
@@ -167,20 +185,63 @@ std::unique_ptr<GSTexture12> GSTexture12::Create(Type type, Format format, int w
 		desc.Flags |= D3D12_RESOURCE_FLAG_ALLOW_UNORDERED_ACCESS;
 
 	wil::com_ptr_nothrow<ID3D12Resource> resource;
+	wil::com_ptr_nothrow<ID3D12Resource> resource_fbl;
 	wil::com_ptr_nothrow<D3D12MA::Allocation> allocation;
-	HRESULT hr = dev->GetAllocator()->CreateResource(&allocationDesc, &desc, state,
-		(type == Type::RenderTarget || type == Type::DepthStencil) ? &optimized_clear_value : nullptr, allocation.put(),
-		IID_PPV_ARGS(resource.put()));
-	if (FAILED(hr))
-	{
-		// OOM isn't fatal.
-		if (hr != E_OUTOFMEMORY)
-			Console.Error("Create texture failed: 0x%08X", hr);
 
-		return {};
+	if (type == Type::RenderTarget)
+	{
+		const D3D12_RESOURCE_ALLOCATION_INFO allocInfo = dev->GetDevice()->GetResourceAllocationInfo(0, 1, &desc);
+
+		HRESULT hr = dev->GetAllocator()->AllocateMemory(&allocationDesc, &allocInfo, allocation.put());
+		if (FAILED(hr))
+		{
+			// OOM isn't fatal.
+			if (hr != E_OUTOFMEMORY)
+				Console.Error("Allocate texture memory failed: 0x%08X", hr);
+
+			return {};
+		}
+
+		hr = dev->GetAllocator()->CreateAliasingResource(allocation.get(), 0, &desc, state,
+			(type == Type::RenderTarget || type == Type::DepthStencil) ? &optimized_clear_value : nullptr,
+			IID_PPV_ARGS(resource.put()));
+		if (FAILED(hr))
+		{
+			// OOM isn't fatal.
+			if (hr != E_OUTOFMEMORY)
+				Console.Error("Create texture resource 1 failed: 0x%08X", hr);
+
+			return {};
+		}
+
+		hr = dev->GetAllocator()->CreateAliasingResource(allocation.get(), 0, &desc, D3D12_RESOURCE_STATE_PIXEL_SHADER_RESOURCE,
+			(type == Type::RenderTarget || type == Type::DepthStencil) ? &optimized_clear_value : nullptr,
+			IID_PPV_ARGS(resource_fbl.put()));
+		if (FAILED(hr))
+		{
+			// OOM isn't fatal.
+			if (hr != E_OUTOFMEMORY)
+				Console.Error("Create texture resource 2 failed: 0x%08X", hr);
+
+			return {};
+		}
+	}
+	else
+	{
+		HRESULT hr = dev->GetAllocator()->CreateResource(&allocationDesc, &desc, state,
+			(type == Type::RenderTarget || type == Type::DepthStencil) ? &optimized_clear_value : nullptr, allocation.put(),
+			IID_PPV_ARGS(resource.put()));
+		if (FAILED(hr))
+		{
+			// OOM isn't fatal.
+			if (hr != E_OUTOFMEMORY)
+				Console.Error("Create texture failed: 0x%08X", hr);
+
+			return {};
+		}
 	}
 
-	D3D12DescriptorHandle srv_descriptor, write_descriptor, uav_descriptor;
+	D3D12DescriptorHandle srv_descriptor, write_descriptor, ro_dsv_descriptor, uav_descriptor, fbl_descriptor;
 	WriteDescriptorType write_descriptor_type = WriteDescriptorType::None;
 	if (srv_format != DXGI_FORMAT_UNKNOWN)
 	{
@@ -195,7 +256,7 @@ std::unique_ptr<GSTexture12> GSTexture12::Create(Type type, Format format, int w
 			write_descriptor_type = WriteDescriptorType::RTV;
 			if (!CreateRTVDescriptor(resource.get(), rtv_format, &write_descriptor))
 			{
-				dev->GetRTVHeapManager().Free(&srv_descriptor);
+				dev->GetDescriptorHeapManager().Free(&srv_descriptor);
 				return {};
 			}
 		}
@@ -204,9 +265,15 @@ std::unique_ptr<GSTexture12> GSTexture12::Create(Type type, Format format, int w
 		case Type::DepthStencil:
 		{
 			write_descriptor_type = WriteDescriptorType::DSV;
-			if (!CreateDSVDescriptor(resource.get(), dsv_format, &write_descriptor))
+			if (!CreateDSVDescriptor(resource.get(), dsv_format, &write_descriptor, false))
 			{
-				dev->GetDSVHeapManager().Free(&srv_descriptor);
+				dev->GetDescriptorHeapManager().Free(&srv_descriptor);
+				return {};
+			}
+			if (!CreateDSVDescriptor(resource.get(), dsv_format, &ro_dsv_descriptor, true))
+			{
+				dev->GetDSVHeapManager().Free(&write_descriptor);
+				dev->GetDescriptorHeapManager().Free(&srv_descriptor);
 				return {};
 			}
 		}
@@ -218,14 +285,47 @@ std::unique_ptr<GSTexture12> GSTexture12::Create(Type type, Format format, int w
 
 	if (uav_format != DXGI_FORMAT_UNKNOWN && !CreateUAVDescriptor(resource.get(), dsv_format, &uav_descriptor))
 	{
-		dev->GetDescriptorHeapManager().Free(&write_descriptor);
+		switch (write_descriptor_type)
+		{
+			case WriteDescriptorType::RTV:
+				dev->GetRTVHeapManager().Free(&write_descriptor);
+				break;
+			case WriteDescriptorType::DSV:
+				dev->GetDSVHeapManager().Free(&ro_dsv_descriptor);
+				dev->GetDSVHeapManager().Free(&write_descriptor);
+				break;
+			default:
+				break;
+		}
 		dev->GetDescriptorHeapManager().Free(&srv_descriptor);
 		return {};
 	}
 
+	if (resource_fbl)
+	{
+		if (!CreateSRVDescriptor(resource_fbl.get(), levels, srv_format, &fbl_descriptor))
+		{
+			dev->GetDescriptorHeapManager().Free(&uav_descriptor);
+			switch (write_descriptor_type)
+			{
+				case WriteDescriptorType::RTV:
+					dev->GetRTVHeapManager().Free(&write_descriptor);
+					break;
+				case WriteDescriptorType::DSV:
+					dev->GetDSVHeapManager().Free(&ro_dsv_descriptor);
+					dev->GetDSVHeapManager().Free(&write_descriptor);
+					break;
+				default:
+					break;
+			}
+			dev->GetDescriptorHeapManager().Free(&srv_descriptor);
+			return {};
+		}
+	}
+
 	return std::unique_ptr<GSTexture12>(
-		new GSTexture12(type, format, width, height, levels, dxgi_format, std::move(resource), std::move(allocation),
-			srv_descriptor, write_descriptor, uav_descriptor, write_descriptor_type, state));
+		new GSTexture12(type, format, width, height, levels, dxgi_format, std::move(resource), std::move(resource_fbl), std::move(allocation),
+			srv_descriptor, write_descriptor, ro_dsv_descriptor, uav_descriptor, fbl_descriptor, write_descriptor_type, state));
 }
 
 std::unique_ptr<GSTexture12> GSTexture12::Adopt(wil::com_ptr_nothrow<ID3D12Resource> resource, Type type, Format format,
@@ -234,7 +334,7 @@ std::unique_ptr<GSTexture12> GSTexture12::Adopt(wil::com_ptr_nothrow<ID3D12Resou
 {
 	const D3D12_RESOURCE_DESC desc = resource->GetDesc();
 
-	D3D12DescriptorHandle srv_descriptor, write_descriptor, uav_descriptor;
+	D3D12DescriptorHandle srv_descriptor, write_descriptor, ro_dsv_descriptor, uav_descriptor;
 	WriteDescriptorType write_descriptor_type = WriteDescriptorType::None;
 	if (srv_format != DXGI_FORMAT_UNKNOWN)
 	{
@@ -247,16 +347,22 @@ std::unique_ptr<GSTexture12> GSTexture12::Adopt(wil::com_ptr_nothrow<ID3D12Resou
 		write_descriptor_type = WriteDescriptorType::RTV;
 		if (!CreateRTVDescriptor(resource.get(), rtv_format, &write_descriptor))
 		{
-			GSDevice12::GetInstance()->GetRTVHeapManager().Free(&srv_descriptor);
+			GSDevice12::GetInstance()->GetDescriptorHeapManager().Free(&srv_descriptor);
 			return {};
 		}
 	}
 	else if (type == Type::DepthStencil)
 	{
 		write_descriptor_type = WriteDescriptorType::DSV;
-		if (!CreateDSVDescriptor(resource.get(), dsv_format, &write_descriptor))
+		if (!CreateDSVDescriptor(resource.get(), dsv_format, &write_descriptor, false))
 		{
-			GSDevice12::GetInstance()->GetDSVHeapManager().Free(&srv_descriptor);
+			GSDevice12::GetInstance()->GetDescriptorHeapManager().Free(&srv_descriptor);
+			return {};
+		}
+		if (!CreateDSVDescriptor(resource.get(), dsv_format, &ro_dsv_descriptor, true))
+		{
+			GSDevice12::GetInstance()->GetDSVHeapManager().Free(&write_descriptor);
+			GSDevice12::GetInstance()->GetDescriptorHeapManager().Free(&srv_descriptor);
 			return {};
 		}
 	}
@@ -265,15 +371,27 @@ std::unique_ptr<GSTexture12> GSTexture12::Adopt(wil::com_ptr_nothrow<ID3D12Resou
 	{
 		if (!CreateUAVDescriptor(resource.get(), srv_format, &uav_descriptor))
 		{
-			GSDevice12::GetInstance()->GetDescriptorHeapManager().Free(&write_descriptor);
+			switch (write_descriptor_type)
+			{
+				case WriteDescriptorType::RTV:
+					GSDevice12::GetInstance()->GetRTVHeapManager().Free(&write_descriptor);
+					break;
+				case WriteDescriptorType::DSV:
+					GSDevice12::GetInstance()->GetDSVHeapManager().Free(&ro_dsv_descriptor);
+					GSDevice12::GetInstance()->GetDSVHeapManager().Free(&write_descriptor);
+					break;
+				default:
+					break;
+			}
+
 			GSDevice12::GetInstance()->GetDescriptorHeapManager().Free(&srv_descriptor);
 			return {};
 		}
 	}
 
 	return std::unique_ptr<GSTexture12>(new GSTexture12(type, format, static_cast<u32>(desc.Width), desc.Height,
-		desc.MipLevels, desc.Format, std::move(resource), {}, srv_descriptor, write_descriptor, uav_descriptor,
-		write_descriptor_type, resource_state));
+		desc.MipLevels, desc.Format, std::move(resource), {}, {}, srv_descriptor, write_descriptor, {}, uav_descriptor,
+		{}, write_descriptor_type, resource_state));
 }
 
 bool GSTexture12::CreateSRVDescriptor(
@@ -297,7 +415,7 @@ bool GSTexture12::CreateRTVDescriptor(ID3D12Resource* resource, DXGI_FORMAT form
 {
 	if (!GSDevice12::GetInstance()->GetRTVHeapManager().Allocate(dh))
 	{
-		Console.Error("Failed to allocate SRV descriptor");
+		Console.Error("Failed to allocate RTV descriptor");
 		return false;
 	}
 
@@ -306,15 +424,15 @@ bool GSTexture12::CreateRTVDescriptor(ID3D12Resource* resource, DXGI_FORMAT form
 	return true;
 }
 
-bool GSTexture12::CreateDSVDescriptor(ID3D12Resource* resource, DXGI_FORMAT format, D3D12DescriptorHandle* dh)
+bool GSTexture12::CreateDSVDescriptor(ID3D12Resource* resource, DXGI_FORMAT format, D3D12DescriptorHandle* dh, bool read_only)
 {
 	if (!GSDevice12::GetInstance()->GetDSVHeapManager().Allocate(dh))
 	{
-		Console.Error("Failed to allocate SRV descriptor");
+		Console.Error("Failed to allocate DSV descriptor");
 		return false;
 	}
 
-	const D3D12_DEPTH_STENCIL_VIEW_DESC desc = {format, D3D12_DSV_DIMENSION_TEXTURE2D, D3D12_DSV_FLAG_NONE};
+	const D3D12_DEPTH_STENCIL_VIEW_DESC desc = {format, D3D12_DSV_DIMENSION_TEXTURE2D, read_only ? D3D12_DSV_FLAG_READ_ONLY_DEPTH : D3D12_DSV_FLAG_NONE };
 	GSDevice12::GetInstance()->GetDevice()->CreateDepthStencilView(resource, &desc, dh->cpu_handle);
 	return true;
 }
@@ -609,7 +727,36 @@ void GSTexture12::TransitionToState(ID3D12GraphicsCommandList* cmdlist, D3D12_RE
 	if (m_resource_state == state)
 		return;
 
-	TransitionSubresourceToState(cmdlist, D3D12_RESOURCE_BARRIER_ALL_SUBRESOURCES, m_resource_state, state);
+	// Read only depth requires special handling as we might want to write stencil.
+	// Also batch the transition barriers as per recommendation from docs.
+	if (state == (D3D12_RESOURCE_STATE_DEPTH_READ | D3D12_RESOURCE_STATE_PIXEL_SHADER_RESOURCE))
+	{
+		// Transition to read depth/write stencil
+		const D3D12_RESOURCE_BARRIER barriers[2] = {
+			{D3D12_RESOURCE_BARRIER_TYPE_TRANSITION, D3D12_RESOURCE_BARRIER_FLAG_NONE,
+				{{m_resource.get(), 0, m_resource_state, (D3D12_RESOURCE_STATE_DEPTH_READ | D3D12_RESOURCE_STATE_PIXEL_SHADER_RESOURCE)}}},
+			{D3D12_RESOURCE_BARRIER_TYPE_TRANSITION, D3D12_RESOURCE_BARRIER_FLAG_NONE,
+				{{m_resource.get(), 1, m_resource_state, D3D12_RESOURCE_STATE_DEPTH_WRITE}}},
+		};
+		GSDevice12::GetInstance()->GetCommandList()->ResourceBarrier(m_resource_state == D3D12_RESOURCE_STATE_DEPTH_WRITE ? 1 : 2, barriers);
+	}
+	else if (m_resource_state == (D3D12_RESOURCE_STATE_DEPTH_READ | D3D12_RESOURCE_STATE_PIXEL_SHADER_RESOURCE))
+	{
+		// Transition from read depth/write stencil
+		const D3D12_RESOURCE_BARRIER barriers[2] = {
+			{D3D12_RESOURCE_BARRIER_TYPE_TRANSITION, D3D12_RESOURCE_BARRIER_FLAG_NONE,
+				{{m_resource.get(), 0, (D3D12_RESOURCE_STATE_DEPTH_READ | D3D12_RESOURCE_STATE_PIXEL_SHADER_RESOURCE), state}}},
+			{D3D12_RESOURCE_BARRIER_TYPE_TRANSITION, D3D12_RESOURCE_BARRIER_FLAG_NONE,
+				{{m_resource.get(), 1, D3D12_RESOURCE_STATE_DEPTH_WRITE, state}}},
+		};
+		GSDevice12::GetInstance()->GetCommandList()->ResourceBarrier(state == D3D12_RESOURCE_STATE_DEPTH_WRITE ? 1 : 2, barriers);
+	}
+	else
+	{
+		// Normal transition
+		TransitionSubresourceToState(cmdlist, D3D12_RESOURCE_BARRIER_ALL_SUBRESOURCES, m_resource_state, state);
+	}
+
 	m_resource_state = state;
 }
 
