@@ -29,23 +29,22 @@ namespace Patch
 	// In PCSX2 it indicates how/when/where the patch line should be applied. If
 	// place is not one of the supported values then the patch line is never applied.
 	// PCSX2 currently supports the following values:
-	// 0 - apply the patch line once on game boot/startup
+	// 0 - apply the patch line once on game boot only
 	// 1 - apply the patch line continuously (technically - on every vsync)
 	// 2 - effect of 0 and 1 combined, see below
+	// 3 - apply the patch line once on game boot or when enabled in the GUI
 	// Note:
 	// - while it may seem that a value of 1 does the same as 0, but also later
 	//   continues to apply the patch on every vsync - it's not.
 	//   The current (and past) behavior is that these patches are applied at different
 	//   places at the code, and it's possible, depending on circumstances, that 0 patches
 	//   will get applied before the first vsync and therefore earlier than 1 patches.
-	// - There's no "place" value which indicates to apply both once on startup
-	//   and then also continuously, however such behavior can be achieved by
-	//   duplicating the line where one has a 0 place and the other has a 1 place.
 	enum patch_place_type : u8
 	{
 		PPT_ONCE_ON_LOAD = 0,
 		PPT_CONTINUOUSLY = 1,
 		PPT_COMBINED_0_1 = 2,
+		PPT_ON_LOAD_OR_WHEN_ENABLED = 3,
 
 		PPT_END_MARKER
 	};
@@ -56,12 +55,14 @@ namespace Patch
 		std::string description;
 		std::string author;
 
+		// This is only populated if all the patch lines in a given group have
+		// the same place value.
+		std::optional<patch_place_type> place;
+
 		std::string_view GetNamePart() const;
 		std::string_view GetNameParentPart() const;
 	};
 
-	using PatchInfoList = std::vector<PatchInfo>;
-
 	struct DynamicPatchEntry
 	{
 		u32 offset;
@@ -80,7 +81,7 @@ namespace Patch
 	extern const char* PATCH_ENABLE_CONFIG_KEY;
 	extern const char* PATCH_DISABLE_CONFIG_KEY;
 
-	extern PatchInfoList GetPatchInfo(const std::string_view serial, u32 crc, bool cheats, bool showAllCRCS, u32* num_unlabelled_patches);
+	extern std::vector<PatchInfo> GetPatchInfo(const std::string_view serial, u32 crc, bool cheats, bool showAllCRCS, u32* num_unlabelled_patches);
 
 	/// Returns the path to a new cheat/patch pnach for the specified serial and CRC.
 	extern std::string GetPnachFilename(const std::string_view serial, u32 crc, bool cheats);
@@ -97,6 +98,13 @@ namespace Patch
 	extern void LoadDynamicPatches(const std::vector<DynamicPatch>& patches);
 	extern void ApplyDynamicPatches(u32 pc);
 
+	// Apply all loaded patches that should be applied when the entry point is
+	// being recompiled.
+	extern void ApplyBootPatches();
+
+	// Apply all loaded patches that should be applied during vsync.
+	extern void ApplyVsyncPatches();
+
 	// Patches the emulation memory by applying all the loaded patches with a specific place value.
 	// Note: unless you know better, there's no need to check whether or not different patch sources
 	// are enabled (e.g. ws patches, auto game fixes, etc) before calling ApplyLoadedPatches,
@@ -105,5 +113,13 @@ namespace Patch
 	// (this happens at AppCoreThread::ApplySettings(...) )
 	extern void ApplyLoadedPatches(patch_place_type place);
 
+	// Get the total counts of the active game patches.
+	extern u32 GetActiveGameDBPatchesCount();
+	extern u32 GetActivePatchesCount();
+	extern u32 GetActiveCheatsCount();
+	extern u32 GetAllActivePatchesCount();
+
 	extern bool IsGloballyToggleablePatch(const PatchInfo& patch_info);
-} // namespace Patch
\ No newline at end of file
+
+	extern const char* PlaceToString(std::optional<patch_place_type> place);
+} // namespace Patch
